#include "snort.hpp"

using namespace std;
using boost::property_tree::ptree;

snort::snort(DDSDomainParticipant *participant) : participant_(participant)
{

    string sysconf_dir(SYSCONFDIR);
    qos_file_ += sysconf_dir + "qos/snort_subscriber_qos.xml";
    cout << qos_file_ << endl;
    
    /* To customize the subscriber QoS, use
       the configuration file USER_QOS_PROFILES.xml */
    subscriber_ = participant_->create_subscriber(
        DDS_SUBSCRIBER_QOS_DEFAULT, NULL /* listener */, DDS_STATUS_MASK_NONE);
    if (subscriber_ == NULL) {
        printf("create_subscriber error\n");
    }

    /* Register the type before creating the topic */
    type_name_ = snort_alertTypeSupport::get_type_name();
    retcode_ = snort_alertTypeSupport::register_type(
        participant_, type_name_);
    if (retcode_ != DDS_RETCODE_OK) {
        printf("register_type error %d\n", retcode_);
    }

    /* To customize the topic QoS, use
       the configuration file USER_QOS_PROFILES.xml */
    topic_ = participant_->create_topic(
        "snort_alert",
        type_name_, DDS_TOPIC_QOS_DEFAULT, NULL /* listener */,
        DDS_STATUS_MASK_NONE);
    if (topic_ == NULL) {
        printf("create_topic error\n");
    }
}


void snort::receive_information(void)
{
    /* Create a data reader listener */
    reader_listener_ = new snort_alertListener();

    /* To customize the data reader QoS, use
       the configuration file USER_QOS_PROFILES.xml */
    reader_ = subscriber_->create_datareader(
        topic_, DDS_DATAREADER_QOS_DEFAULT, reader_listener_,
        DDS_STATUS_MASK_ALL);
    if (reader_ == NULL) {
        printf("create_datareader error\n");
        delete reader_listener_;
    }

}

snort_alertListener::snort_alertListener()
{

    string sysconf_dir(SYSCONFDIR);
    string filename(sysconf_dir + "cavecanem_subscriber.xml");
    
	try {
		ptree pt;
		read_xml(filename,pt);
    	//Reading database data
        database_server_ = pt.get<string>("database.server");
    	database_name_ = pt.get<string>("database.name");
    	database_user_ = pt.get<string>("database.user");
    	database_password_ = pt.get<string>("database.password");
    }
    
    catch(std::exception &e) {
		cerr << e.what() << endl;
	}

	conn_.connect(
	    database_name_.c_str(), 
	    database_server_.c_str(), 
	    database_user_.c_str(),
	    database_password_.c_str());
	    
	mysqlpp::Query query = conn_.query();
	
    query << "CREATE TABLE IF NOT EXISTS hosts (" << 
             "hostname VARCHAR(30) NOT NULL PRIMARY KEY," <<
             "sysname VARCHAR(30)," <<
             "release_name VARCHAR(30)," <<
             "machine VARCHAR(30))";
    query.execute();

//	query.exec("DROP TABLE IF EXISTS snort");

	query << "CREATE TABLE IF NOT EXISTS snort (" <<
			"snort_key VARCHAR(30) NOT NULL," <<
			"hostname VARCHAR(30) NOT NULL," <<
			"snort_ts TIMESTAMP NOT NULL," <<
			"msg VARCHAR(100) NOT NULL," <<
			"source_network_addr VARCHAR(15) NOT NULL," <<
			"source_port INT UNSIGNED," <<
			"source_mac VARCHAR(30) NOT NULL," <<
			"target_network_addr VARCHAR(15) NOT NULL," <<
			"target_port INT UNSIGNED," <<
			"target_mac VARCHAR(30) NOT NULL," <<
			"PRIMARY KEY(snort_key)," <<
			"FOREIGN KEY(hostname) REFERENCES hosts(hostname))";
	query.execute();

}

void snort_alertListener::on_data_available(DDSDataReader* reader)
{
    snort_alertDataReader *snort_alert_reader = NULL;
    snort_alertSeq data_seq;
    DDS_SampleInfoSeq info_seq;
    DDS_ReturnCode_t retcode;
    int i;

    snort_alert_reader = snort_alertDataReader::narrow(reader);
    if (snort_alert_reader == NULL) {
        printf("DataReader narrow error\n");
        return;
    }

    retcode = snort_alert_reader->take(
        data_seq, info_seq, DDS_LENGTH_UNLIMITED,
        DDS_ANY_SAMPLE_STATE, DDS_ANY_VIEW_STATE, DDS_ANY_INSTANCE_STATE);

    if (retcode == DDS_RETCODE_NO_DATA) {
        return;
    } else if (retcode != DDS_RETCODE_OK) {
        printf("take error %d\n", retcode);
        return;
    }

    for (i = 0; i < data_seq.length(); ++i) {
        if (info_seq[i].valid_data) {

			mysqlpp::Query query = conn_.query();

            map<char *,host_info>::iterator hostinfo_map_it;
            hostinfo_map_it = hostinfo_map_.find(data_seq[i].host.hostname);
            //Dealing with hosts table
            if(hostinfo_map_it == hostinfo_map_.end()) {
                hostinfo_map_[data_seq[i].host.hostname] = data_seq[i].host;
	            query << "REPLACE INTO hosts VALUES (" <<
	                    "'" << data_seq[i].host.hostname << "'," << 
	                    "'" << data_seq[i].host.sysname << "'," <<
	                    "'" << data_seq[i].host.release_name << "'," <<
	                    "'" << data_seq[i].host.machine << "')";
                query.execute();
            }    


			query << "REPLACE INTO snort VALUES ("
				<< "'" << data_seq[i].timestamp_string << "',"
				<< "'" << data_seq[i].host.hostname << "'," //hostname
        		<< "'" << data_seq[i].ts.year << "-"
        		<< data_seq[i].ts.mon << "-"
        		<< data_seq[i].ts.mday << " "
        		<< data_seq[i].ts.hour << ":"
        		<< data_seq[i].ts.min << ":"
        		<< data_seq[i].ts.sec << "',"
        		<< "'" << data_seq[i].msg  << "',"
        		<< "'" << data_seq[i].source.network_addr << "',"
        		<< data_seq[i].source.prt << ","
        		<< "'" << data_seq[i].source.mac << "',"
        		<< "'" << data_seq[i].target.network_addr << "',"
        		<< data_seq[i].target.prt << ","
        		<< "'" << data_seq[i].target.mac << "')";
			query.execute();
        }
    }

    retcode = snort_alert_reader->return_loan(data_seq, info_seq);
    if (retcode != DDS_RETCODE_OK) {
        printf("return loan error %d\n", retcode);
    }

}

